-- name=int_window
--******************************************************************************************
function public.Init( pos_z )  

  private.list = {};
  private.position = {};
  private.event_anim_end = {};

  private.SHOW_ANIM_TIME  = 0.30;
  private.BLACKBACK_ALPHA = 0.80;
  private.SHADE_RGB       = 0.10;

  private.hub = "int_window_hub";
  private.blackback_name = "spr_int_window_blackback";

  ObjCreate( private.hub, "obj" );
  ObjSet( private.hub, { pos_z = pos_z } );
  ObjAttach( private.hub, interface.originhub );

end;
--******************************************************************************************
function public.Destroy()  
end;
--******************************************************************************************
function public.Show( message_params )

  local window_name    = message_params.window_name;
  local window_pos_beg = message_params.window_pos_beg;
  local window_pos_end = message_params.window_pos_end;

  local module_path = message_params.window_module_path;
  local module_name = message_params.window_module_name;

  if _G[ module_name ] == nil then

    ModLoad( module_path );
    _G[ module_name ].Init();

  end;

  if message_params.func then

    message_params.func();

  end;

  local is_already_show = false;
 
  for i = 1, #private.list do

    if ( private.list[ i ] == window_name ) then

      --DbgTrace( "Dialog window < "..dlgname.." > already on the screen!" );
      is_already_show = true;
      break;

    end;

  end;

  if ( not is_already_show ) then
  
    ObjSet( window_name, { input = false } );
    ObjAttach( window_name, private.hub );

    table.insert( private.list, window_name );

    if ( #private.list == 1 ) then

      ObjSet( private.blackback_name, { active = true, visible = true, input = true } );
      ObjAttach( private.blackback_name, private.hub );

      local event_id = "show_blackback";
      private.event_anim_end[ event_id ] = {};
      private.BlackBackShowAnim( event_id );

    else

      local window_shade = private.list[ #private.list - 1 ];
      
      ObjSet( window_shade, { input = false } );

      local event_id = "shade_"..window_shade;
      private.event_anim_end[ event_id ] = {};
      private.ShadeAnim( window_shade, event_id );

    end;

    private.position[ window_name ] = window_pos_beg;

    local event_id = "show_"..window_name;
    private.event_anim_end[ event_id ] = window_name;
    private.ShowAnim( window_name, window_pos_beg, window_pos_end, event_id );

  end;

end;
--******************************************************************************************
function public.Hide( message_params )

  local window_name    = message_params.window_name;
  local window_pos_beg = { ObjGet( window_name ).pos_x, ObjGet( window_name ).pos_y };
  local window_pos_end = private.position[ window_name ];

  ObjSet( window_name, { input = false } );

  table.remove( private.list );

  if ( #private.list == 0 ) then

    local event_id = "hide_blackback";
    private.event_anim_end[ event_id ] = {};
    private.BlackBackHideAnim( event_id );

  else

    local window_bright = private.list[ #private.list ];

    local event_id = "bright_"..window_bright;
    private.event_anim_end[ event_id ] = window_bright;
    private.BrightAnim( window_bright, event_id );

  end;

  local event_id = "hide_"..window_name;
  private.event_anim_end[ event_id ] = window_name;
  private.HideAnim( window_name, window_pos_beg, window_pos_end, event_id );

end;
--******************************************************************************************
function private.BlackBackShowAnim( event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;
  local alp = private.BLACKBACK_ALPHA;

  ObjAnimate( private.blackback_name, "alp", 0, 0, trg,
  {
    ( 0 * tme ), 0, 0,
    ( 1 * tme ), 0, alp
  } );

end;
--******************************************************************************************
function private.BlackBackHideAnim( event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;
  local alp = private.BLACKBACK_ALPHA;

  ObjAnimate( private.blackback_name, "alp", 0, 0, trg,
  {
  ( 0 * tme ), 0, alp,
  ( 1 * tme ), 0, 0
  } );

end;
--******************************************************************************************
function private.ShadeAnim( obj_name, event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;
  local rgb = private.SHADE_RGB;

  ObjAnimate( obj_name, "color_rgb", 0, 0, trg,
  {
  ( 0 * tme ), 0, 1, 1, 1,
  ( 1 * tme ), 0, rgb, rgb, rgb
  });

end;
--******************************************************************************************
function private.BrightAnim( obj_name, event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;
  local rgb = private.SHADE_RGB;

  ObjAnimate( obj_name, "color_rgb", 0, 0, trg,
  {
  ( 0 * tme ), 0, rgb, rgb, rgb,
  ( 1 * tme ), 0, 1, 1, 1
  } );

end;
--******************************************************************************************
function private.ShowAnim( obj_name, pos_beg, pos_end, event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;

  ObjAnimate( obj_name, "pos_xy", 0, 0, trg,
  {
  ( 0.0 * tme ), 0, pos_beg[ 1 ], pos_beg[ 2 ],
  ( 1.0 * tme ), 0, pos_end[ 1 ], pos_end[ 2 ],
  } );
  ObjAnimate( obj_name, "scale_xy", 0, 0, "",
  {
  ( 0.0 * tme ), 0, 0, 0,
  ( 1.0 * tme ), 0, 1, 1
  } );
  ObjAnimate( obj_name, "alp", 0, 0, "",
  {
  ( 0.0 * tme ), 2, 0,
  ( 0.5 * tme ), 2, 0,
  ( 1.0 * tme ), 2, 1 
  });

end;
--******************************************************************************************
function private.HideAnim( obj_name, pos_beg, pos_end, event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;

  ObjAnimate( obj_name, "pos_xy", 0, 0, trg,
  {
  ( 0.0 * tme ), 0, pos_beg[ 1 ], pos_beg[ 2 ],
  ( 1.0 * tme ), 0, pos_end[ 1 ], pos_end[ 2 ],
  } );
  ObjAnimate( obj_name, "scale_xy", 0, 0, "",
  {
  ( 0.0 * tme ), 0, 1, 1,
  ( 1.0 * tme ), 0, 0, 0
  } );
  ObjAnimate( obj_name, "alp", 0, 0, "",
  {
  ( 0.0 * tme ), 1, 1,
  ( 0.5 * tme ), 1, 0,
  ( 1.0 * tme ), 1, 0
  } );

end;
--******************************************************************************************
function private.EventAnimEnd( event_id )

  local event_params = private.event_anim_end[ event_id ];

------------------------------------------------------------------------------------
  if ( event_id == "show_blackback" ) then

    --

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "shade" ) ) then

    --

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "show" ) ) then

    local wnd_name = event_params;
    ObjSet( wnd_name, { input = true } );

------------------------------------------------------------------------------------
  elseif ( event_id == "hide_blackback" ) then

    ObjSet( private.blackback_name, { active = false, visible = false, input = false } );
    ObjDetach( private.blackback_name );

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "bright" ) ) then

    local wnd_name = event_params;
    ObjSet( wnd_name, { input = true } );

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "hide" ) ) then

    local wnd_name = event_params;
    ObjDetach( wnd_name );

------------------------------------------------------------------------------------
  end;

end;
--******************************************************************************************
--******************************************************************************************
function public.GetIntShown()
  
  --DbgTrace("GetIntShown() cmn.is_inmenunow:"..tostring(cmn.is_inmenunow));
  if #private.list>0 then
    
    local name = string.reverse(private.list[ #private.list ]);
    name = string.sub(name,1,string.find(name,"_")-1);
    name = string.reverse(name);
    
    --DbgTrace("GetIntShown() #private.list>0 name:"..tostring(name));
    
    (( common.dialog[ name ] or {} ).Close or function ()
      --DbgTrace("public.GetIntShown():Hide");
        public.Hide( { window_name = private.list[ #private.list ] } ); 
      end )();
    
    if #private.list == 0 then 
      PauseLevel( 0 );
    end;
    --DbgTrace("GetIntShown() return true;");
    return true;
  elseif game == nil then
    return false;
  elseif game.room_names ~= nil then
    --DbgTrace("GetIntShown() game.room_names ~= nil");
    if int_complex_inv.IsOnScreen() then
    --int_complex_inv or int_deploy_inv -- fix this game
      --DbgTrace("int_complex_inv.IsOnScreen() return true;");
      int_complex_inv.Hide();
      return true;
    elseif int_strategy_guide.isActive() then
      --DbgTrace("int_strategy_guide.isActive() return true;");
      int_strategy_guide.Hide();
      return true;
    elseif common_impl.is_in_map == 1 then
      --DbgTrace("common_impl.is_in_map == 1 return true;");
      local loc = GetCurrentRoom();
      if loc == "rm_diary"  then
        rm_diary.Hide();
      elseif loc == "rm_collectibles" then
        rm_collectibles.Hide();
      elseif loc == "rm_achievements" then
        rm_achievements.Hide();
      else
        int_map.Close();
      end;
      return true;
    elseif common_impl.show_hint.dialog~=nil then
      --DbgTrace("common_impl.show_hint.dialog~=nil return true;");
      --interface_impl.DialogHintHide(); fix this game
      --interface.DialogHintHide()
      --int.DialogHintHide()
      interface.DialogHintHide()();
      return true;
    else
      local func = (int_tutorial or {} ).GetTutorialActive or false;
      local check = false;
      if func then
        if func() == true then
          check = true;
        end;
      end;
      if check then
        --DbgTrace("GetIntShown() return true; tutorial hide");
        return true;
      else  
        --DbgTrace("GetIntShown() return false;");
        return false;
      end;
    end;
  end;
    
end;
--******************************************************************************************